@@ -228,13 +228,9 @@ impl Expression for ArithmeticExpression {
     }
 
     fn expr_type(&self, scope: &Environment) -> DataType {
-        let lhs = self.left.expr_type(scope);
-        let rhs = self.left.expr_type(scope);
-
-        if lhs.is_int() && rhs.is_int() {
+        if self.left.expr_type(scope).is_int() && self.right.expr_type(scope).is_int() {
             return DataType::Integer;
         }
-
         DataType::Float
     }
 
@@ -437,6 +433,7 @@ pub struct InExpression {
     pub argument: Box<dyn Expression>,
     pub values: Vec<Box<dyn Expression>>,
     pub values_type: DataType,
+    pub has_not_keyword: bool,
 }
 
 impl Expression for InExpression {
