@@ -3,6 +3,7 @@ use crate::types::DataType;
 use crate::value::Value;
 
 use lazy_static::lazy_static;
+use std::cmp::Ordering;
 use std::collections::HashMap;
 
 type Function = fn(&[Value]) -> Value;
@@ -39,6 +40,7 @@ lazy_static! {
         map.insert("soundex", text_soundex);
         map.insert("concat", text_concat);
         map.insert("unicode", text_unicode);
+        map.insert("strcmp", text_strcmp);
 
         // Date functions
         map.insert("current_date", date_current_date);
@@ -49,6 +51,8 @@ lazy_static! {
         map.insert("maketime", date_make_time);
         map.insert("dayname", date_dayname);
         map.insert("monthname", date_monthname);
+        map.insert("hour", date_hour);
+        map.insert("isdate", date_is_date);
 
         // Numeric functions
         map.insert("abs", numeric_abs);
@@ -69,6 +73,8 @@ lazy_static! {
         map.insert("isnull", general_is_null);
         map.insert("isnumeric", general_is_numeric);
         map.insert("typeof", general_type_of);
+        map.insert("greatest", general_greatest);
+        map.insert("least", general_least);
         map
     };
 }
@@ -238,6 +244,7 @@ lazy_static! {
                 result: DataType::Integer
              },
         );
+        map.insert("strcmp", Prototype { parameters: vec![DataType::Text, DataType::Text], result: DataType::Integer });
 
         // Date functions
         map.insert(
@@ -296,6 +303,20 @@ lazy_static! {
                 result: DataType::Text,
             }
         );
+        map.insert(
+            "hour",
+            Prototype {
+                parameters: vec![DataType::DateTime],
+                result: DataType::Integer,
+            }
+        );
+        map.insert(
+            "isdate",
+            Prototype {
+                parameters: vec![DataType::Any],
+                result: DataType::Boolean,
+            }
+        );
         // Numeric functions
         map.insert(
             "abs",
@@ -410,6 +431,20 @@ lazy_static! {
                 result: DataType::Text,
             },
         );
+        map.insert(
+            "greatest",
+            Prototype {
+                parameters: vec![DataType::Any, DataType::Any, DataType::Varargs(Box::new(DataType::Any))],
+                result: DataType::Any
+             },
+        );
+        map.insert(
+            "least",
+            Prototype {
+                parameters: vec![DataType::Any, DataType::Any, DataType::Varargs(Box::new(DataType::Any))],
+                result: DataType::Any
+             },
+        );
         map
     };
 }
@@ -646,6 +681,14 @@ fn text_concat(inputs: &[Value]) -> Value {
     Value::Text(text.concat())
 }
 
+fn text_strcmp(inputs: &[Value]) -> Value {
+    Value::Integer(match inputs[0].as_text().cmp(&inputs[1].as_text()) {
+        std::cmp::Ordering::Less => 1,
+        std::cmp::Ordering::Equal => 2,
+        std::cmp::Ordering::Greater => 0,
+    })
+}
+
 // Date functions
 
 fn date_current_date(_inputs: &[Value]) -> Value {
@@ -690,6 +733,16 @@ fn date_monthname(inputs: &[Value]) -> Value {
     Value::Text(month_str)
 }
 
+fn date_hour(inputs: &[Value]) -> Value {
+    let date = inputs[0].as_date_time();
+    let hour = date_utils::date_time_to_hour(date);
+    Value::Integer(hour)
+}
+
+fn date_is_date(inputs: &[Value]) -> Value {
+    Value::Boolean(inputs[0].data_type().is_date())
+}
+
 // Numeric functions
 
 fn numeric_abs(inputs: &[Value]) -> Value {
@@ -779,3 +832,27 @@ fn general_type_of(inputs: &[Value]) -> Value {
     let input_type = inputs[0].data_type();
     Value::Text(input_type.to_string())
 }
+
+fn general_greatest(inputs: &[Value]) -> Value {
+    let mut max = &inputs[0];
+
+    for value in inputs.iter().skip(1) {
+        if max.compare(value) == Ordering::Greater {
+            max = value;
+        }
+    }
+
+    max.to_owned()
+}
+
+fn general_least(inputs: &[Value]) -> Value {
+    let mut least = &inputs[0];
+
+    for value in inputs.iter().skip(1) {
+        if least.compare(value) == Ordering::Less {
+            least = value;
+        }
+    }
+
+    least.to_owned()
+}
