@@ -26,7 +26,7 @@ pub enum Query {
 }
 
 pub struct GQLQuery {
-    pub statements: HashMap<String, Box<dyn Statement>>,
+    pub statements: HashMap<&'static str, Box<dyn Statement>>,
     pub has_aggregation_function: bool,
     pub has_group_by_statement: bool,
     pub hidden_selections: Vec<String>,
