@@ -45,9 +45,9 @@ impl Value {
             DataType::Integer => self.as_int() == other.as_int(),
             DataType::Float => self.as_float() == other.as_float(),
             DataType::Boolean => self.as_bool() == other.as_bool(),
-            DataType::DateTime => self.as_date() == other.as_date(),
+            DataType::DateTime => self.as_date_time() == other.as_date_time(),
             DataType::Date => self.as_date() == other.as_date(),
-            DataType::Time => self.as_date() == other.as_date(),
+            DataType::Time => self.as_time() == other.as_time(),
             DataType::Undefined => true,
             DataType::Null => true,
             _ => false,
