@@ -2,6 +2,8 @@ use gitql_ast::environment::Environment;
 use gitql_ast::environment::TABLES_FIELDS_NAMES;
 use gitql_ast::value::Value;
 use std::collections::HashMap;
+use std::num::IntErrorKind;
+use std::num::ParseIntError;
 use std::vec;
 
 use crate::context::ParserContext;
@@ -23,59 +25,78 @@ use gitql_ast::statement::*;
 use gitql_ast::types::DataType;
 use gitql_ast::types::TABLES_FIELDS_TYPES;
 
-pub fn parse_gql(mut tokens: Vec<Token>, env: &mut Environment) -> Result<Query, Box<Diagnostic>> {
-    consume_optional_semicolon_if_exists(&mut tokens);
-
+pub fn parse_gql(tokens: Vec<Token>, env: &mut Environment) -> Result<Query, Box<Diagnostic>> {
     let mut position = 0;
     let first_token = &tokens[position];
-    match &first_token.kind {
-        TokenKind::Set => parse_set_query(env, &tokens),
-        TokenKind::Select => parse_select_query(env, &tokens),
+    let query_result = match &first_token.kind {
+        TokenKind::Set => parse_set_query(env, &tokens, &mut position),
+        TokenKind::Select => parse_select_query(env, &tokens, &mut position),
         _ => Err(un_expected_statement_error(&tokens, &mut position)),
+    };
+
+    // Consume optional `;` at the end of valid statement
+    if let Some(last_token) = tokens.get(position) {
+        if last_token.kind == TokenKind::Semicolon {
+            position += 1;
+        }
     }
+
+    // Check for un expected content after valid statement
+    if query_result.is_ok() && position < tokens.len() {
+        return Err(un_expected_content_after_correct_statement(
+            &first_token.literal,
+            &tokens,
+            &mut position,
+        ));
+    }
+
+    query_result
 }
 
-fn parse_set_query(env: &mut Environment, tokens: &Vec<Token>) -> Result<Query, Box<Diagnostic>> {
+fn parse_set_query(
+    env: &mut Environment,
+    tokens: &Vec<Token>,
+    position: &mut usize,
+) -> Result<Query, Box<Diagnostic>> {
     let len = tokens.len();
-    let mut position = 0;
     let mut context = ParserContext::default();
 
     // Consume Set keyword
-    position += 1;
+    *position += 1;
 
-    if position >= len || tokens[position].kind != TokenKind::GlobalVariable {
+    if *position >= len || tokens[*position].kind != TokenKind::GlobalVariable {
         return Err(Diagnostic::error(
             "Expect Global variable name start with `@` after `SET` keyword",
         )
-        .with_location(get_safe_location(tokens, position - 1))
+        .with_location(get_safe_location(tokens, *position - 1))
         .as_boxed());
     }
 
-    let name = &tokens[position].literal;
+    let name = &tokens[*position].literal;
 
     // Consume variable name
-    position += 1;
+    *position += 1;
 
-    if position >= len || !is_assignment_operator(&tokens[position]) {
+    if *position >= len || !is_assignment_operator(&tokens[*position]) {
         return Err(
             Diagnostic::error("Expect `=` or `:=` and Value after Variable name")
-                .with_location(get_safe_location(tokens, position - 1))
+                .with_location(get_safe_location(tokens, *position - 1))
                 .as_boxed(),
         );
     }
 
     // Consume `=` or `:=` token
-    position += 1;
+    *position += 1;
 
     let aggregations_count_before = context.aggregations.len();
-    let value = parse_expression(&mut context, env, tokens, &mut position)?;
+    let value = parse_expression(&mut context, env, tokens, position)?;
     let has_aggregations = context.aggregations.len() != aggregations_count_before;
 
     // Until supports sub queries, aggregation value can't be stored in variables
     if has_aggregations {
         return Err(
             Diagnostic::error("Aggregation value can't be assigned to global variable")
-                .with_location(get_safe_location(tokens, position - 1))
+                .with_location(get_safe_location(tokens, *position - 1))
                 .as_boxed(),
         );
     }
@@ -91,15 +112,15 @@ fn parse_set_query(env: &mut Environment, tokens: &Vec<Token>) -> Result<Query,
 fn parse_select_query(
     env: &mut Environment,
     tokens: &Vec<Token>,
+    position: &mut usize,
 ) -> Result<Query, Box<Diagnostic>> {
     let len = tokens.len();
-    let mut position = 0;
 
     let mut context = ParserContext::default();
-    let mut statements: HashMap<String, Box<dyn Statement>> = HashMap::new();
+    let mut statements: HashMap<&'static str, Box<dyn Statement>> = HashMap::new();
 
-    while position < len {
-        let token = &tokens[position];
+    while *position < len {
+        let token = &tokens[*position];
 
         match &token.kind {
             TokenKind::Select => {
@@ -109,8 +130,8 @@ fn parse_select_query(
                         .with_location(token.location)
                         .as_boxed());
                 }
-                let statement = parse_select_statement(&mut context, env, tokens, &mut position)?;
-                statements.insert("select".to_string(), statement);
+                let statement = parse_select_statement(&mut context, env, tokens, position)?;
+                statements.insert("select", statement);
                 context.is_single_value_query = !context.aggregations.is_empty();
             }
             TokenKind::Where => {
@@ -121,8 +142,8 @@ fn parse_select_query(
                         .as_boxed());
                 }
 
-                let statement = parse_where_statement(&mut context, env, tokens, &mut position)?;
-                statements.insert("where".to_string(), statement);
+                let statement = parse_where_statement(&mut context, env, tokens, position)?;
+                statements.insert("where", statement);
             }
             TokenKind::Group => {
                 if statements.contains_key("group") {
@@ -132,8 +153,8 @@ fn parse_select_query(
                         .as_boxed());
                 }
 
-                let statement = parse_group_by_statement(&mut context, env, tokens, &mut position)?;
-                statements.insert("group".to_string(), statement);
+                let statement = parse_group_by_statement(&mut context, env, tokens, position)?;
+                statements.insert("group", statement);
             }
             TokenKind::Having => {
                 if statements.contains_key("having") {
@@ -154,8 +175,8 @@ fn parse_select_query(
                     .as_boxed());
                 }
 
-                let statement = parse_having_statement(&mut context, env, tokens, &mut position)?;
-                statements.insert("having".to_string(), statement);
+                let statement = parse_having_statement(&mut context, env, tokens, position)?;
+                statements.insert("having", statement);
             }
             TokenKind::Limit => {
                 if statements.contains_key("limit") {
@@ -165,8 +186,65 @@ fn parse_select_query(
                         .as_boxed());
                 }
 
-                let statement = parse_limit_statement(tokens, &mut position)?;
-                statements.insert("limit".to_string(), statement);
+                let statement = parse_limit_statement(tokens, position)?;
+                statements.insert("limit", statement);
+
+                // Check for Limit and Offset shortcut
+                if *position < len && tokens[*position].kind == TokenKind::Comma {
+                    // Prevent user from using offset statement more than one time
+                    if statements.contains_key("offset") {
+                        return Err(Diagnostic::error("You already used `OFFSET` statement")
+                            .add_note(
+                                "Can't use more than one `OFFSET` statement in the same query",
+                            )
+                            .with_location(token.location)
+                            .as_boxed());
+                    }
+
+                    // Consume Comma
+                    *position += 1;
+
+                    if *position >= len || tokens[*position].kind != TokenKind::Integer {
+                        return Err(Diagnostic::error(
+                            "Expects `OFFSET` amount as Integer value after `,`",
+                        )
+                        .add_help("Try to add constant Integer after comma")
+                        .add_note("`OFFSET` value must be a constant Integer")
+                        .with_location(token.location)
+                        .as_boxed());
+                    }
+
+                    let count_result: Result<usize, ParseIntError> =
+                        tokens[*position].literal.parse();
+
+                    // Report clear error for Integer parsing
+                    if let Err(error) = &count_result {
+                        if error.kind().eq(&IntErrorKind::PosOverflow) {
+                            return Err(Diagnostic::error("`OFFSET` integer value is too large")
+                                .add_help("Try to use smaller value")
+                                .add_note(&format!(
+                                    "`OFFSET` value must be between 0 and {}",
+                                    usize::MAX
+                                ))
+                                .with_location(token.location)
+                                .as_boxed());
+                        }
+
+                        return Err(Diagnostic::error("`OFFSET` integer value is invalid")
+                            .add_help(&format!(
+                                "`OFFSET` value must be between 0 and {}",
+                                usize::MAX
+                            ))
+                            .with_location(token.location)
+                            .as_boxed());
+                    }
+
+                    // Consume Offset value
+                    *position += 1;
+
+                    let count = count_result.unwrap();
+                    statements.insert("offset", Box::new(OffsetStatement { count }));
+                }
             }
             TokenKind::Offset => {
                 if statements.contains_key("offset") {
@@ -176,8 +254,8 @@ fn parse_select_query(
                         .as_boxed());
                 }
 
-                let statement = parse_offset_statement(tokens, &mut position)?;
-                statements.insert("offset".to_string(), statement);
+                let statement = parse_offset_statement(tokens, position)?;
+                statements.insert("offset", statement);
             }
             TokenKind::Order => {
                 if statements.contains_key("order") {
@@ -187,10 +265,10 @@ fn parse_select_query(
                         .as_boxed());
                 }
 
-                let statement = parse_order_by_statement(&mut context, env, tokens, &mut position)?;
-                statements.insert("order".to_string(), statement);
+                let statement = parse_order_by_statement(&mut context, env, tokens, position)?;
+                statements.insert("order", statement);
             }
-            _ => return Err(un_expected_statement_error(tokens, &mut position)),
+            _ => break,
         }
     }
 
@@ -199,7 +277,7 @@ fn parse_select_query(
         let aggregation_functions = AggregationsStatement {
             aggregations: context.aggregations,
         };
-        statements.insert("aggregation".to_string(), Box::new(aggregation_functions));
+        statements.insert("aggregation", Box::new(aggregation_functions));
     }
 
     // Remove all selected fields from hidden selection
@@ -229,6 +307,8 @@ fn parse_select_statement(
 
     if *position >= tokens.len() {
         return Err(Diagnostic::error("Incomplete input for select statement")
+            .add_help("Try select one or more values in the `SELECT` statement")
+            .add_note("Select statements requires at least selecting one value")
             .with_location(get_safe_location(tokens, *position - 1))
             .as_boxed());
     }
@@ -287,6 +367,7 @@ fn parse_select_statement(
                 {
                     return Err(
                         Diagnostic::error("You already have field with the same name")
+                            .add_help("Try to use a new unique name for alias")
                             .with_location(get_safe_location(tokens, *position))
                             .as_boxed(),
                     );
@@ -326,6 +407,7 @@ fn parse_select_statement(
         let table_name_token = consume_kind(tokens, *position, TokenKind::Symbol);
         if table_name_token.is_err() {
             return Err(Diagnostic::error("Expect `identifier` as a table name")
+                .add_note("Table name must be an identifier")
                 .with_location(get_safe_location(tokens, *position))
                 .as_boxed());
         }
@@ -336,6 +418,7 @@ fn parse_select_statement(
         table_name = &table_name_token.ok().unwrap().literal;
         if !TABLES_FIELDS_NAMES.contains_key(table_name) {
             return Err(Diagnostic::error("Unresolved table name")
+                .add_help("Check the documentations to see available tables")
                 .with_location(get_safe_location(tokens, *position))
                 .as_boxed());
         }
@@ -347,6 +430,7 @@ fn parse_select_statement(
     if is_select_all && table_name.is_empty() {
         return Err(
             Diagnostic::error("Expect `FROM` and table name after `SELECT *`")
+                .add_note("Select all must be used with valid table name")
                 .with_location(get_safe_location(tokens, *position))
                 .as_boxed(),
         );
@@ -355,6 +439,8 @@ fn parse_select_statement(
     // Select input validations
     if !is_select_all && fields_names.is_empty() {
         return Err(Diagnostic::error("Incomplete input for select statement")
+            .add_help("Try select one or more values in the `SELECT` statement")
+            .add_note("Select statements requires at least selecting one value")
             .with_location(get_safe_location(tokens, *position - 1))
             .as_boxed());
     }
@@ -369,7 +455,7 @@ fn parse_select_statement(
         );
     }
 
-    // Type check all selected fields has type regsited in type table
+    // Type check all selected fields has type registered in type table
     type_check_selected_fields(env, table_name, &fields_names, tokens, *position)?;
 
     Ok(Box::new(SelectStatement {
@@ -390,6 +476,8 @@ fn parse_where_statement(
     *position += 1;
     if *position >= tokens.len() {
         return Err(Diagnostic::error("Expect expression after `WHERE` keyword")
+            .add_help("Try to add boolean expression after `WHERE` keyword")
+            .add_note("`WHERE` statement expects expression as condition")
             .with_location(get_safe_location(tokens, *position - 1))
             .as_boxed());
     }
@@ -406,6 +494,7 @@ fn parse_where_statement(
             DataType::Boolean,
             condition_type
         ))
+        .add_note("`WHERE` statement condition must be Boolean")
         .with_location(condition_location)
         .as_boxed());
     }
@@ -414,6 +503,8 @@ fn parse_where_statement(
     if aggregations_count_before != aggregations_count_after {
         return Err(
             Diagnostic::error("Can't use Aggregation functions in `WHERE` statement")
+                .add_note("Aggregation functions must be used after `GROUP BY` statement")
+                .add_note("Aggregation functions evaluated after later after `GROUP BY` statement")
                 .with_location(condition_location)
                 .as_boxed(),
         );
@@ -432,6 +523,7 @@ fn parse_group_by_statement(
     if *position >= tokens.len() || tokens[*position].kind != TokenKind::By {
         return Err(
             Diagnostic::error("Expect keyword `by` after keyword `group`")
+                .add_help("Try to use `BY` keyword after `GROUP")
                 .with_location(get_safe_location(tokens, *position - 1))
                 .as_boxed(),
         );
@@ -449,6 +541,7 @@ fn parse_group_by_statement(
     if !env.contains(&field_name) {
         return Err(
             Diagnostic::error("Current table not contains field with this name")
+                .add_help("Check the documentations to see available fields for each tables")
                 .with_location(get_safe_location(tokens, *position - 1))
                 .as_boxed(),
         );
@@ -466,9 +559,13 @@ fn parse_having_statement(
 ) -> Result<Box<dyn Statement>, Box<Diagnostic>> {
     *position += 1;
     if *position >= tokens.len() {
-        return Err(Diagnostic::error("Expect expression after `where` keyword")
-            .with_location(get_safe_location(tokens, *position - 1))
-            .as_boxed());
+        return Err(
+            Diagnostic::error("Expect expression after `HAVING` keyword")
+                .add_help("Try to add boolean expression after `HAVING` keyword")
+                .add_note("`HAVING` statement expects expression as condition")
+                .with_location(get_safe_location(tokens, *position - 1))
+                .as_boxed(),
+        );
     }
 
     // Make sure HAVING condition expression has boolean type
@@ -481,6 +578,7 @@ fn parse_having_statement(
             DataType::Boolean,
             condition_type
         ))
+        .add_note("`HAVING` statement condition must be Boolean")
         .with_location(condition_location)
         .as_boxed());
     }
@@ -499,9 +597,34 @@ fn parse_limit_statement(
             .as_boxed());
     }
 
-    let count_str = tokens[*position].literal.to_string();
-    let count: usize = count_str.parse().unwrap();
+    let count_result: Result<usize, ParseIntError> = tokens[*position].literal.parse();
+
+    // Report clear error for Integer parsing
+    if let Err(error) = &count_result {
+        if error.kind().eq(&IntErrorKind::PosOverflow) {
+            return Err(Diagnostic::error("`LIMIT` integer value is too large")
+                .add_help("Try to use smaller value")
+                .add_note(&format!(
+                    "`LIMIT` value must be between 0 and {}",
+                    usize::MAX
+                ))
+                .with_location(get_safe_location(tokens, *position))
+                .as_boxed());
+        }
+
+        return Err(Diagnostic::error("`LIMIT` integer value is invalid")
+            .add_help(&format!(
+                "`LIMIT` value must be between 0 and {}",
+                usize::MAX
+            ))
+            .with_location(get_safe_location(tokens, *position))
+            .as_boxed());
+    }
+
+    // Consume Integer value
     *position += 1;
+
+    let count = count_result.unwrap();
     Ok(Box::new(LimitStatement { count }))
 }
 
@@ -516,9 +639,33 @@ fn parse_offset_statement(
             .as_boxed());
     }
 
-    let count_str = tokens[*position].literal.to_string();
-    let count: usize = count_str.parse().unwrap();
+    let count_result: Result<usize, ParseIntError> = tokens[*position].literal.parse();
+
+    // Report clear error for Integer parsing
+    if let Err(error) = &count_result {
+        if error.kind().eq(&IntErrorKind::PosOverflow) {
+            return Err(Diagnostic::error("`OFFSET` integer value is too large")
+                .add_help("Try to use smaller value")
+                .add_note(&format!(
+                    "`OFFSET` value must be between 0 and {}",
+                    usize::MAX
+                ))
+                .with_location(get_safe_location(tokens, *position))
+                .as_boxed());
+        }
+
+        return Err(Diagnostic::error("`OFFSET` integer value is invalid")
+            .add_help(&format!(
+                "`OFFSET` value must be between 0 and {}",
+                usize::MAX
+            ))
+            .with_location(get_safe_location(tokens, *position))
+            .as_boxed());
+    }
+
     *position += 1;
+
+    let count = count_result.unwrap();
     Ok(Box::new(OffsetStatement { count }))
 }
 
@@ -534,6 +681,7 @@ fn parse_order_by_statement(
     if *position >= tokens.len() || tokens[*position].kind != TokenKind::By {
         return Err(
             Diagnostic::error("Expect keyword `BY` after keyword `ORDER")
+                .add_help("Try to use `BY` keyword after `ORDER")
                 .with_location(get_safe_location(tokens, *position - 1))
                 .as_boxed(),
         );
@@ -588,6 +736,11 @@ fn parse_expression(
         let column_name = context.generate_column_name();
         env.define(column_name.to_string(), expression.expr_type(env));
 
+        // Register the new aggregation generated field if the this expression is after group by
+        if context.has_group_by_statement && !context.hidden_selections.contains(&column_name) {
+            context.hidden_selections.push(column_name.to_string());
+        }
+
         context
             .aggregations
             .insert(column_name.clone(), AggregateValue::Expression(expression));
@@ -660,18 +813,18 @@ fn parse_is_null_expression(
         if *position < tokens.len() && tokens[*position].kind == TokenKind::Null {
             // Consume `Null` keyword
             *position += 1;
-        } else {
-            return Err(
-                Diagnostic::error("Expects `NULL` Keyword after `IS` or `IS NOT`")
-                    .with_location(is_location)
-                    .as_boxed(),
-            );
+
+            return Ok(Box::new(IsNullExpression {
+                argument: expression,
+                has_not: has_not_keyword,
+            }));
         }
 
-        return Ok(Box::new(IsNullExpression {
-            argument: expression,
-            has_not: has_not_keyword,
-        }));
+        return Err(
+            Diagnostic::error("Expects `NULL` Keyword after `IS` or `IS NOT`")
+                .with_location(is_location)
+                .as_boxed(),
+        );
     }
     Ok(expression)
 }
@@ -683,6 +836,15 @@ fn parse_in_expression(
     position: &mut usize,
 ) -> Result<Box<dyn Expression>, Box<Diagnostic>> {
     let expression = parse_between_expression(context, env, tokens, position)?;
+
+    // Consume `NOT` keyword if IN Expression prefixed with `NOT` for example `expr NOT IN (...values)`
+    let has_not_keyword = if *position < tokens.len() && tokens[*position].kind == TokenKind::Not {
+        *position += 1;
+        true
+    } else {
+        false
+    };
+
     if *position < tokens.len() && tokens[*position].kind == TokenKind::In {
         let in_location = tokens[*position].location;
 
@@ -698,6 +860,14 @@ fn parse_in_expression(
         }
 
         let values = parse_arguments_expressions(context, env, tokens, position)?;
+
+        // Optimize the Expression if the number of values in the list is 0
+        if values.is_empty() {
+            return Ok(Box::new(BooleanExpression {
+                is_true: has_not_keyword,
+            }));
+        }
+
         let values_type_result = check_all_values_are_same_type(env, &values);
         if values_type_result.is_none() {
             return Err(Diagnostic::error(
@@ -721,8 +891,22 @@ fn parse_in_expression(
             argument: expression,
             values,
             values_type,
+            has_not_keyword,
         }));
     }
+
+    // Report error if user write `NOT` with no `IN` keyword after it
+    if has_not_keyword {
+        return Err(
+            Diagnostic::error("Expects `IN` expression after this `NOT` keyword")
+                .add_help("Try to use `IN` expression after NOT keyword")
+                .add_help("Try to remove `NOT` keyword")
+                .add_note("Expect to see `NOT` then `IN` keyword with a list of values")
+                .with_location(get_safe_location(tokens, *position - 1))
+                .as_boxed(),
+        );
+    }
+
     Ok(expression)
 }
 
@@ -740,48 +924,37 @@ fn parse_between_expression(
         // Consume `BETWEEN` keyword
         *position += 1;
 
-        if expression.expr_type(env) != DataType::Integer {
-            return Err(Diagnostic::error(&format!(
-                "BETWEEN value must to be Number type but got {}",
-                expression.expr_type(env)
-            ))
-            .with_location(between_location)
-            .as_boxed());
-        }
-
         if *position >= tokens.len() {
             return Err(
-                Diagnostic::error("Between keyword expects two range after it")
+                Diagnostic::error("`BETWEEN` keyword expects two range after it")
                     .with_location(between_location)
                     .as_boxed(),
             );
         }
 
+        let argument_type = expression.expr_type(env);
         let range_start = parse_logical_or_expression(context, env, tokens, position)?;
-        if range_start.expr_type(env) != DataType::Integer {
-            return Err(Diagnostic::error(&format!(
-                "Expect range start to be Number type but got {}",
-                range_start.expr_type(env)
-            ))
-            .with_location(between_location)
-            .as_boxed());
-        }
 
         if *position >= tokens.len() || tokens[*position].kind != TokenKind::DotDot {
-            return Err(Diagnostic::error("Expect `..` after BETWEEN range start")
+            return Err(Diagnostic::error("Expect `..` after `BETWEEN` range start")
                 .with_location(between_location)
                 .as_boxed());
         }
 
-        // Consume `..` keyword
+        // Consume `..` token
         *position += 1;
 
         let range_end = parse_logical_or_expression(context, env, tokens, position)?;
-        if range_end.expr_type(env) != DataType::Integer {
+
+        if argument_type != range_start.expr_type(env) || argument_type != range_end.expr_type(env)
+        {
             return Err(Diagnostic::error(&format!(
-                "Expect range end to be Number type but got {}",
+                "Expect `BETWEEN` argument, range start and end to has same type but got {}, {} and {}",
+                argument_type,
+                range_start.expr_type(env),
                 range_end.expr_type(env)
             ))
+            .add_help("Try to make sure all of them has same type")
             .with_location(between_location)
             .as_boxed());
         }
@@ -808,10 +981,7 @@ fn parse_logical_or_expression(
     }
 
     let lhs = expression.ok().unwrap();
-
-    let operator = &tokens[*position];
-
-    if operator.kind == TokenKind::LogicalOr {
+    if tokens[*position].kind == TokenKind::LogicalOr {
         *position += 1;
 
         if lhs.expr_type(env) != DataType::Boolean {
@@ -855,10 +1025,7 @@ fn parse_logical_and_expression(
     }
 
     let lhs = expression.ok().unwrap();
-
-    let operator = &tokens[*position];
-
-    if operator.kind == TokenKind::LogicalAnd {
+    if tokens[*position].kind == TokenKind::LogicalAnd {
         *position += 1;
 
         if lhs.expr_type(env) != DataType::Boolean {
@@ -902,10 +1069,7 @@ fn parse_bitwise_or_expression(
     }
 
     let lhs = expression.ok().unwrap();
-
-    let operator = &tokens[*position];
-
-    if operator.kind == TokenKind::BitwiseOr {
+    if tokens[*position].kind == TokenKind::BitwiseOr {
         *position += 1;
 
         if lhs.expr_type(env) != DataType::Boolean {
@@ -949,10 +1113,7 @@ fn parse_logical_xor_expression(
     }
 
     let lhs = expression.ok().unwrap();
-
-    let operator = &tokens[*position];
-
-    if operator.kind == TokenKind::LogicalXor {
+    if tokens[*position].kind == TokenKind::LogicalXor {
         *position += 1;
 
         if lhs.expr_type(env) != DataType::Boolean {
@@ -994,10 +1155,7 @@ fn parse_bitwise_and_expression(
     }
 
     let lhs = expression.ok().unwrap();
-
-    let operator = &tokens[*position];
-
-    if operator.kind == TokenKind::BitwiseAnd {
+    if tokens[*position].kind == TokenKind::BitwiseAnd {
         *position += 1;
 
         if lhs.expr_type(env) != DataType::Boolean {
@@ -1008,16 +1166,7 @@ fn parse_bitwise_and_expression(
             ));
         }
 
-        let right_expr = parse_equality_expression(context, env, tokens, position);
-        if right_expr.is_err() {
-            return Err(
-                Diagnostic::error("Can't parser right side of bitwise and expression")
-                    .with_location(get_safe_location(tokens, *position))
-                    .as_boxed(),
-            );
-        }
-
-        let rhs = right_expr.ok().unwrap();
+        let rhs = parse_equality_expression(context, env, tokens, position)?;
         if rhs.expr_type(env) != DataType::Boolean {
             return Err(type_mismatch_error(
                 tokens[*position].location,
@@ -1238,11 +1387,24 @@ fn parse_term_expression(
             continue;
         }
 
+        // Report Error message that suggest to replace `+` operator by `CONCAT` function
+        if math_operator == ArithmeticOperator::Plus {
+            return Err(Diagnostic::error(&format!(
+                "Math operators `+` both sides to be number types but got `{}` and `{}`",
+                lhs_type, rhs_type
+            ))
+            .add_help(
+                "You can use `CONCAT(Any, Any, ...Any)` function to concatenate values with different types",
+            )
+            .with_location(operator.location)
+            .as_boxed());
+        }
+
         return Err(Diagnostic::error(&format!(
             "Math operators require number types but got `{}` and `{}`",
             lhs_type, rhs_type
         ))
-        .with_location(get_safe_location(tokens, *position - 2))
+        .with_location(operator.location)
         .as_boxed());
     }
 
@@ -1487,6 +1649,8 @@ fn parse_function_call_expression(
             let argument_result = get_expression_name(&arguments[0]);
             if argument_result.is_err() {
                 return Err(Diagnostic::error("Invalid Aggregation function argument")
+                    .add_help("Try to use field name as Aggregation function argument")
+                    .add_note("Aggregation function accept field name as argument")
                     .with_location(function_name_location)
                     .as_boxed());
             }
@@ -1546,15 +1710,16 @@ fn parse_arguments_expressions(
             }
         }
 
-        if consume_kind(tokens, *position, TokenKind::RightParen).is_ok() {
-            *position += 1;
-        } else {
+        if consume_kind(tokens, *position, TokenKind::RightParen).is_err() {
             return Err(
                 Diagnostic::error("Expect `)` after function call arguments")
+                    .add_help("Try to add ')' at the end of function call, after arguments")
                     .with_location(get_safe_location(tokens, *position))
                     .as_boxed(),
             );
         }
+
+        *position += 1;
     }
     Ok(arguments)
 }
@@ -1631,6 +1796,7 @@ fn parse_group_expression(
     if tokens[*position].kind != TokenKind::RightParen {
         return Err(Diagnostic::error("Expect `)` to end group expression")
             .with_location(get_safe_location(tokens, *position))
+            .add_help("Try to add ')' at the end of group expression")
             .as_boxed());
     }
     *position += 1;
@@ -1659,12 +1825,13 @@ fn parse_case_expression(
             if has_else_branch {
                 return Err(
                     Diagnostic::error("This case expression already has else branch")
+                        .add_note("`CASE` expression can has only one `ELSE` branch")
                         .with_location(get_safe_location(tokens, *position))
                         .as_boxed(),
                 );
             }
 
-            // consume else keyword
+            // Consume `ELSE` keyword
             *position += 1;
 
             let default_value_expr = parse_expression(context, env, tokens, position)?;
@@ -1673,15 +1840,16 @@ fn parse_case_expression(
             continue;
         }
 
-        // When
+        // Check if current token kind is `WHEN` keyword
         let when_result = consume_kind(tokens, *position, TokenKind::When);
         if when_result.is_err() {
             return Err(Diagnostic::error("Expect `when` before case condition")
+                .add_help("Try to add `WHEN` keyword before any condition")
                 .with_location(get_safe_location(tokens, *position))
                 .as_boxed());
         }
 
-        // Consume when keyword
+        // Consume `WHEN` keyword
         *position += 1;
 
         let condition = parse_expression(context, env, tokens, position)?;
@@ -1695,7 +1863,7 @@ fn parse_case_expression(
 
         let then_result = consume_kind(tokens, *position, TokenKind::Then);
         if then_result.is_err() {
-            return Err(Diagnostic::error("Expect `then` after case condition")
+            return Err(Diagnostic::error("Expect `THEN` after case condition")
                 .with_location(get_safe_location(tokens, *position))
                 .as_boxed());
         }
@@ -1718,7 +1886,7 @@ fn parse_case_expression(
 
     // Make sure case expression end with END keyword
     if *position >= tokens.len() || tokens[*position].kind != TokenKind::End {
-        return Err(Diagnostic::error("Expect `end` after case branches")
+        return Err(Diagnostic::error("Expect `END` after case branches")
             .with_location(get_safe_location(tokens, *position))
             .as_boxed());
     }
@@ -1741,6 +1909,7 @@ fn parse_case_expression(
                 "Case value in branch {} has different type than the last branch",
                 i + 1
             ))
+            .add_note("All values in `CASE` expression must has the same Type")
             .with_location(case_location)
             .as_boxed());
         }
@@ -1842,11 +2011,11 @@ fn check_function_call_arguments(
             }
             TypeCheckResult::NotEqualAndCantImplicitCast => {
                 let argument_type = argument.expr_type(env);
-                return Err(Box::new(Diagnostic::error(&format!(
+                return Err(Diagnostic::error(&format!(
                     "Function `{}` argument number {} with type `{}` don't match expected type `{}`",
                     function_name, index, argument_type, parameter_type
                 ))
-                .with_location(location)));
+                .with_location(location).as_boxed());
             }
             _ => {}
         }
@@ -1868,11 +2037,11 @@ fn check_function_call_arguments(
                 TypeCheckResult::NotEqualAndCantImplicitCast => {
                     let argument_type = arguments.get(index).unwrap().expr_type(env);
                     if !last_parameter_type.eq(&argument_type) {
-                        return Err(Box::new(Diagnostic::error(&format!(
+                        return Err(Diagnostic::error(&format!(
                             "Function `{}` argument number {} with type `{}` don't match expected type `{}`",
                             function_name, index, argument_type, last_parameter_type
                         ))
-                        .with_location(location)));
+                        .with_location(location).as_boxed());
                     }
                 }
                 _ => {}
@@ -1901,21 +2070,32 @@ fn type_check_selected_fields(
             continue;
         }
 
-        return Err(Box::new(
-            Diagnostic::error(&format!(
-                "Table `{}` has no field with name `{}`",
-                table_name, field_name
-            ))
-            .with_location(get_safe_location(tokens, position)),
-        ));
+        return Err(Diagnostic::error(&format!(
+            "Table `{}` has no field with name `{}`",
+            table_name, field_name
+        ))
+        .add_help("Check the documentations to see available fields for each tables")
+        .with_location(get_safe_location(tokens, position))
+        .as_boxed());
     }
     Ok(())
 }
 
-fn un_expected_statement_error(tokens: &Vec<Token>, position: &mut usize) -> Box<Diagnostic> {
+fn un_expected_statement_error(tokens: &[Token], position: &mut usize) -> Box<Diagnostic> {
+    let token: &Token = &tokens[*position];
+    let location = token.location;
+
+    // Query starts with invalid statement
+    if location.start == 0 {
+        return Diagnostic::error("Unexpected statement")
+            .add_help("Expect query to start with `SELECT` or `SET` keyword")
+            .with_location(location)
+            .as_boxed();
+    }
+
+    // General un expected statement error
     Diagnostic::error("Unexpected statement")
-        .add_help("Expect query to start with `SELECT` or `SET` keyword")
-        .with_location(get_safe_location(tokens, *position))
+        .with_location(location)
         .as_boxed()
 }
 
@@ -1986,18 +2166,28 @@ fn un_expected_expression_error(tokens: &Vec<Token>, position: &usize) -> Box<Di
         .as_boxed()
 }
 
-/// Remove last token if it semicolon, because it's optional
-#[inline(always)]
-fn consume_optional_semicolon_if_exists(tokens: &mut Vec<Token>) {
-    if tokens.is_empty() {
-        return;
-    }
+/// Report error message for extra content after the end of current statement
+fn un_expected_content_after_correct_statement(
+    statement_name: &str,
+    tokens: &Vec<Token>,
+    position: &mut usize,
+) -> Box<Diagnostic> {
+    let error_message = &format!(
+        "Unexpected content after the end of `{}` statement",
+        statement_name.to_uppercase()
+    );
 
-    if let Some(last_token) = tokens.last() {
-        if last_token.kind == TokenKind::Semicolon {
-            tokens.remove(tokens.len() - 1);
-        }
-    }
+    // The range of extra content
+    let location_of_extra_content = Location {
+        start: tokens[*position].location.start,
+        end: tokens[tokens.len() - 1].location.end,
+    };
+
+    Diagnostic::error(error_message)
+        .add_help("Try to check if statement keyword is missing")
+        .add_help("Try remove un expected extra content")
+        .with_location(location_of_extra_content)
+        .as_boxed()
 }
 
 #[allow(clippy::borrowed_box)]
