@@ -1,5 +1,6 @@
 #[derive(PartialEq)]
 pub enum TokenKind {
+    Set,
     Select,
     Distinct,
     From,
@@ -49,6 +50,7 @@ pub enum TokenKind {
     BitwiseLeftShift,
 
     Symbol,
+    GlobalVariable,
     Integer,
     Float,
     String,
@@ -57,6 +59,8 @@ pub enum TokenKind {
     False,
     Null,
 
+    ColonEqual,
+
     Plus,
     Minus,
     Star,
@@ -65,6 +69,7 @@ pub enum TokenKind {
 
     Comma,
     Dot,
+    Semicolon,
 
     Ascending,
     Descending,
@@ -82,9 +87,9 @@ pub struct Token {
     pub literal: String,
 }
 
-use crate::diagnostic::GQLError;
+use crate::diagnostic::Diagnostic;
 
-pub fn tokenize(script: String) -> Result<Vec<Token>, GQLError> {
+pub fn tokenize(script: String) -> Result<Vec<Token>, Box<Diagnostic>> {
     let mut tokens: Vec<Token> = Vec::new();
 
     let mut position = 0;
@@ -100,8 +105,21 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, GQLError> {
 
         // Symbol
         if char.is_alphabetic() {
-            let identifier = consume_identifier(&characters, &mut position, &mut column_start);
-            tokens.push(identifier);
+            tokens.push(consume_identifier(
+                &characters,
+                &mut position,
+                &mut column_start,
+            ));
+            continue;
+        }
+
+        // Global Variable Symbol
+        if char == '@' {
+            tokens.push(consume_global_variable_name(
+                &characters,
+                &mut position,
+                &mut column_start,
+            )?);
             continue;
         }
 
@@ -111,39 +129,62 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, GQLError> {
                 if characters[position + 1] == 'x' {
                     position += 2;
                     column_start += 2;
-                    let result = consume_hex_number(&characters, &mut position, &mut column_start)?;
-                    tokens.push(result);
+                    tokens.push(consume_hex_number(
+                        &characters,
+                        &mut position,
+                        &mut column_start,
+                    )?);
                     continue;
                 }
 
                 if characters[position + 1] == 'b' {
                     position += 2;
                     column_start += 2;
-                    let result =
-                        consume_binary_number(&characters, &mut position, &mut column_start)?;
-                    tokens.push(result);
+                    tokens.push(consume_binary_number(
+                        &characters,
+                        &mut position,
+                        &mut column_start,
+                    )?);
                     continue;
                 }
 
                 if characters[position + 1] == 'o' {
                     position += 2;
                     column_start += 2;
-                    let result =
-                        consume_octal_number(&characters, &mut position, &mut column_start)?;
-                    tokens.push(result);
+                    tokens.push(consume_octal_number(
+                        &characters,
+                        &mut position,
+                        &mut column_start,
+                    )?);
                     continue;
                 }
             }
 
-            let number = consume_number(&characters, &mut position, &mut column_start)?;
-            tokens.push(number);
+            tokens.push(consume_number(
+                &characters,
+                &mut position,
+                &mut column_start,
+            )?);
             continue;
         }
 
         // String literal
         if char == '"' {
-            let result = consume_string(&characters, &mut position, &mut column_start)?;
-            tokens.push(result);
+            tokens.push(consume_string(
+                &characters,
+                &mut position,
+                &mut column_start,
+            )?);
+            continue;
+        }
+
+        // All chars between two backticks should be consumed as identifier
+        if char == '`' {
+            tokens.push(consume_backticks_identifier(
+                &characters,
+                &mut position,
+                &mut column_start,
+            )?);
             continue;
         }
 
@@ -211,10 +252,7 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, GQLError> {
         if char == '/' {
             // Ignore C style comment which from /* comment */
             if position + 1 < characters.len() && characters[position + 1] == '*' {
-                let result = ignore_c_style_comment(&characters, &mut position);
-                if result.is_err() {
-                    return Err(result.err().unwrap());
-                }
+                ignore_c_style_comment(&characters, &mut position)?;
                 continue;
             }
 
@@ -463,6 +501,32 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, GQLError> {
             continue;
         }
 
+        // Colon Equal
+        if char == ':' {
+            if position + 1 < len && characters[position + 1] == '=' {
+                let location = Location {
+                    start: column_start,
+                    end: position,
+                };
+
+                let token = Token {
+                    location,
+                    kind: TokenKind::ColonEqual,
+                    literal: ":=".to_owned(),
+                };
+
+                tokens.push(token);
+                position += 2;
+                continue;
+            }
+
+            return Err(Box::new(
+                Diagnostic::error("Expect `=` after `:`")
+                    .add_help("Only token that has `:` is `:=` so make sure you add `=` after `:`")
+                    .with_location_span(column_start, position),
+            ));
+        }
+
         // Bang or Bang Equal
         if char == '!' {
             let location = Location {
@@ -527,30 +591,83 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, GQLError> {
             continue;
         }
 
+        // Semicolon
+        if char == ';' {
+            let location = Location {
+                start: column_start,
+                end: position,
+            };
+
+            let token = Token {
+                location,
+                kind: TokenKind::Semicolon,
+                literal: ";".to_owned(),
+            };
+
+            tokens.push(token);
+            position += 1;
+            continue;
+        }
+
         // Characters to ignoring
         if char == ' ' || char == '\n' || char == '\t' {
             position += 1;
             continue;
         }
 
-        return Err(GQLError {
-            message: "Un expected character".to_owned(),
-            location: Location {
-                start: column_start,
-                end: position,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("Un expected character").with_location_span(column_start, position),
+        ));
     }
 
     Ok(tokens)
 }
 
-fn consume_identifier(chars: &Vec<char>, pos: &mut usize, start: &mut usize) -> Token {
-    while *pos < chars.len() && (chars[*pos] == '_' || chars[*pos].is_alphabetic()) {
+fn consume_global_variable_name(
+    chars: &Vec<char>,
+    pos: &mut usize,
+    start: &mut usize,
+) -> Result<Token, Box<Diagnostic>> {
+    // Consume `@`
+    *pos += 1;
+
+    // Make sure first character is  alphabetic
+    if *pos < chars.len() && !chars[*pos].is_alphabetic() {
+        return Err(Box::new(
+            Diagnostic::error("Global variable name must start with alphabetic character")
+                .add_help("Add at least one alphabetic character after @")
+                .with_location_span(*start, *pos),
+        ));
+    }
+
+    while *pos < chars.len() && (chars[*pos] == '_' || chars[*pos].is_alphanumeric()) {
         *pos += 1;
     }
 
-    // Idntifier is be case-insensitive by default, convert to lowercase to be easy to compare and lookup
+    // Identifier is be case-insensitive by default, convert to lowercase to be easy to compare and lookup
+    let literal = &chars[*start..*pos];
+    let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect())
+        .unwrap()
+        .to_lowercase();
+
+    let location = Location {
+        start: *start,
+        end: *pos,
+    };
+
+    Ok(Token {
+        location,
+        kind: TokenKind::GlobalVariable,
+        literal: string,
+    })
+}
+
+fn consume_identifier(chars: &Vec<char>, pos: &mut usize, start: &mut usize) -> Token {
+    while *pos < chars.len() && (chars[*pos] == '_' || chars[*pos].is_alphanumeric()) {
+        *pos += 1;
+    }
+
+    // Identifier is be case-insensitive by default, convert to lowercase to be easy to compare and lookup
     let literal = &chars[*start..*pos];
     let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect())
         .unwrap()
@@ -572,7 +689,7 @@ fn consume_number(
     chars: &Vec<char>,
     pos: &mut usize,
     start: &mut usize,
-) -> Result<Token, GQLError> {
+) -> Result<Token, Box<Diagnostic>> {
     let mut kind = TokenKind::Integer;
 
     while *pos < chars.len() && (chars[*pos].is_numeric() || chars[*pos] == '_') {
@@ -604,11 +721,49 @@ fn consume_number(
     })
 }
 
+fn consume_backticks_identifier(
+    chars: &Vec<char>,
+    pos: &mut usize,
+    start: &mut usize,
+) -> Result<Token, Box<Diagnostic>> {
+    *pos += 1;
+
+    while *pos < chars.len() && chars[*pos] != '`' {
+        *pos += 1;
+    }
+
+    if *pos >= chars.len() {
+        return Err(Box::new(
+            Diagnostic::error("Unterminated backticks")
+                .add_help("Add ` at the end of the identifier")
+                .with_location_span(*start, *pos),
+        ));
+    }
+
+    *pos += 1;
+
+    let literal = &chars[*start + 1..*pos - 1];
+    let identifier = String::from_utf8(literal.iter().map(|&c| c as u8).collect()).unwrap();
+
+    let location = Location {
+        start: *start,
+        end: *pos,
+    };
+
+    let string_literal = Token {
+        location,
+        kind: TokenKind::Symbol,
+        literal: identifier,
+    };
+
+    Ok(string_literal)
+}
+
 fn consume_binary_number(
     chars: &Vec<char>,
     pos: &mut usize,
     start: &mut usize,
-) -> Result<Token, GQLError> {
+) -> Result<Token, Box<Diagnostic>> {
     let mut has_digit = false;
     while *pos < chars.len() && ((chars[*pos] == '0' || chars[*pos] == '1') || chars[*pos] == '_') {
         *pos += 1;
@@ -616,13 +771,12 @@ fn consume_binary_number(
     }
 
     if !has_digit {
-        return Err(GQLError {
-            message: "Missing digits after the integer base prefix".to_owned(),
-            location: Location {
-                start: *start,
-                end: *pos,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("Missing digits after the integer base prefix")
+                .add_help("Expect at least one binary digits after the prefix 0b")
+                .add_help("Binary digit mean 0 or 1")
+                .with_location_span(*start, *pos),
+        ));
     }
 
     let literal = &chars[*start..*pos];
@@ -631,13 +785,9 @@ fn consume_binary_number(
     let convert_result = i64::from_str_radix(&literal_num, 2);
 
     if convert_result.is_err() {
-        return Err(GQLError {
-            message: "Invalid binary number".to_owned(),
-            location: Location {
-                start: *start,
-                end: *pos,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("Invalid binary number").with_location_span(*start, *pos),
+        ));
     }
 
     let location = Location {
@@ -656,7 +806,7 @@ fn consume_octal_number(
     chars: &Vec<char>,
     pos: &mut usize,
     start: &mut usize,
-) -> Result<Token, GQLError> {
+) -> Result<Token, Box<Diagnostic>> {
     let mut has_digit = false;
     while *pos < chars.len() && ((chars[*pos] >= '0' || chars[*pos] < '8') || chars[*pos] == '_') {
         *pos += 1;
@@ -664,13 +814,12 @@ fn consume_octal_number(
     }
 
     if !has_digit {
-        return Err(GQLError {
-            message: "Missing digits after the integer base prefix".to_owned(),
-            location: Location {
-                start: *start,
-                end: *pos,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("Missing digits after the integer base prefix")
+                .add_help("Expect at least one octal digits after the prefix 0o")
+                .add_help("Octal digit mean 0 to 8 number")
+                .with_location_span(*start, *pos),
+        ));
     }
 
     let literal = &chars[*start..*pos];
@@ -679,13 +828,9 @@ fn consume_octal_number(
     let convert_result = i64::from_str_radix(&literal_num, 8);
 
     if convert_result.is_err() {
-        return Err(GQLError {
-            message: "Invalid octal number".to_owned(),
-            location: Location {
-                start: *start,
-                end: *pos,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("Invalid octal number").with_location_span(*start, *pos),
+        ));
     }
 
     let location = Location {
@@ -704,7 +849,7 @@ fn consume_hex_number(
     chars: &Vec<char>,
     pos: &mut usize,
     start: &mut usize,
-) -> Result<Token, GQLError> {
+) -> Result<Token, Box<Diagnostic>> {
     let mut has_digit = false;
     while *pos < chars.len() && (chars[*pos].is_ascii_hexdigit() || chars[*pos] == '_') {
         *pos += 1;
@@ -712,13 +857,12 @@ fn consume_hex_number(
     }
 
     if !has_digit {
-        return Err(GQLError {
-            message: "Missing digits after the integer base prefix".to_owned(),
-            location: Location {
-                start: *start,
-                end: *pos,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("Missing digits after the integer base prefix")
+                .add_help("Expect at least one hex digits after the prefix 0x")
+                .add_help("Hex digit mean 0 to 9 and a to f")
+                .with_location_span(*start, *pos),
+        ));
     }
 
     let literal = &chars[*start..*pos];
@@ -727,13 +871,9 @@ fn consume_hex_number(
     let convert_result = i64::from_str_radix(&literal_num, 16);
 
     if convert_result.is_err() {
-        return Err(GQLError {
-            message: "Invalid hex decinmal number".to_owned(),
-            location: Location {
-                start: *start,
-                end: *pos,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("Invalid hex decimal number").with_location_span(*start, *pos),
+        ));
     }
 
     let location = Location {
@@ -752,7 +892,7 @@ fn consume_string(
     chars: &Vec<char>,
     pos: &mut usize,
     start: &mut usize,
-) -> Result<Token, GQLError> {
+) -> Result<Token, Box<Diagnostic>> {
     *pos += 1;
 
     while *pos < chars.len() && chars[*pos] != '"' {
@@ -760,13 +900,11 @@ fn consume_string(
     }
 
     if *pos >= chars.len() {
-        return Err(GQLError {
-            message: "Unterminated double quote string".to_owned(),
-            location: Location {
-                start: *start,
-                end: *pos,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("Unterminated double quote string")
+                .add_help("Add \" at the end of the String literal")
+                .with_location_span(*start, *pos),
+        ));
     }
 
     *pos += 1;
@@ -798,7 +936,7 @@ fn ignore_single_line_comment(chars: &Vec<char>, pos: &mut usize) {
     *pos += 1;
 }
 
-fn ignore_c_style_comment(chars: &Vec<char>, pos: &mut usize) -> Result<(), GQLError> {
+fn ignore_c_style_comment(chars: &Vec<char>, pos: &mut usize) -> Result<(), Box<Diagnostic>> {
     *pos += 2;
 
     while *pos + 1 < chars.len() && (chars[*pos] != '*' && chars[*pos + 1] != '/') {
@@ -806,13 +944,11 @@ fn ignore_c_style_comment(chars: &Vec<char>, pos: &mut usize) -> Result<(), GQLE
     }
 
     if *pos + 2 > chars.len() {
-        return Err(GQLError {
-            message: "C Style comment must end with */".to_owned(),
-            location: Location {
-                start: *pos,
-                end: *pos,
-            },
-        });
+        return Err(Box::new(
+            Diagnostic::error("C Style comment must end with */")
+                .add_help("Add */ at the end of C Style comments")
+                .with_location_span(*pos, *pos),
+        ));
     }
 
     *pos += 2;
@@ -822,6 +958,7 @@ fn ignore_c_style_comment(chars: &Vec<char>, pos: &mut usize) -> Result<(), GQLE
 fn resolve_symbol_kind(literal: String) -> TokenKind {
     match literal.to_lowercase().as_str() {
         // Reserved keywords
+        "set" => TokenKind::Set,
         "select" => TokenKind::Select,
         "distinct" => TokenKind::Distinct,
         "from" => TokenKind::From,
